/* 
 * Copyright (c) 2003,2004  Daniel Bryan
 * All rights reserved.
 *
 * For more information see COPYRIGHT.
 */

#define BLACK     1
#define RED       2
#define GREEN	  3
#define YELLOW    4
#define BLUE	  5
#define MAGENTA	  6
#define CYAN	  7
#define WHITE     8
#define BLACK2    9
#define RED2	 10
#define GREEN2	 11
#define YELLOW2  12
#define BLUE2	 13
#define MAGENTA2 14
#define CYAN2    15
#define WHITE2	 16

#define USE_BRIGHT 9

void coloron(WINDOW *win);
void coloroff(WINDOW *win);
